package com.caimei365.user.controller;

import com.caimei365.user.idempotent.Idempotent;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.model.dto.ClubRegisterDto;
import com.caimei365.user.model.dto.ClubUpgradeDto;
import com.caimei365.user.model.dto.ShopRegisterDto;
import com.caimei365.user.model.po.ClubPo;
import com.caimei365.user.model.po.ShopPo;
import com.caimei365.user.service.RegisterService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 注册Api
 *
 * @author : Charles
 * @date : 2021/3/10
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/user/register")
public class RegisterApi {
    private final RegisterService registerService;

    /**
     * 普通机构入驻(注册)
     *
     * spi旧接口：/club/common
     *
     * @param clubRegisterDto ClubRegisterDto{
     *                          source            注册来源: 0网站 1小程序
     *                          userName          用户名
     *                          bindMobile        企业绑定手机号
     *                          password          密码
     *                          unionId           微信unionId
     *                          nickName          微信昵称
     *                          avatarUrl         微信头像(旧：headimgurl)
     *                        }
     * @param passWordConfirm 用户确认密码
     * @param smsCode         短信验证码(旧：activationCode)
     * @param isAgreed        是否同意勾选同意协议，1是，其他否
     * @param headers         HttpHeaders
     * @return ClubPo
     */
    @Idempotent(prefix="idempotent_club", keys={"#baseUser"}, expire=5)
    @PostMapping("/club")
    public ResponseJson<ClubPo> clubRegister(ClubRegisterDto clubRegisterDto,
                                             String passWordConfirm,
                                             String smsCode,
                                             Integer isAgreed,
                                             @RequestHeader HttpHeaders headers) {
        return registerService.clubRegister(clubRegisterDto, passWordConfirm, smsCode, isAgreed, headers);
    }

    /**
     * 普通机构升级会员机构
     *
     * spi旧接口：/club/upgrade
     *
     * @param upgradeDto ClubUpgradeDto:{
     *                      clubId                 机构ID
     *                      userId                 用户ID
     *                      name                   机构名称
     *                      sName                  机构简称
     *                      contractEmail          邮箱（contractEmail1）
     *                      contractPhone          固定电话
     *                      linkMan                联系人（linkMan1）
     *                      provinceId             省Id
     *                      cityId                 市Id
     *                      townId                 县区Id
     *                      address                地址
     *                      shopPhoto              门头照(headpic)
     *                      businessLicense        营业执照（businessLicenseImage）
     *                      socialCreditCode       统一社会信用代码（socialCreditCode）
     *                      firstClubType          一级分类：医美=1和生美=2
     *                      secondClubType         医美的二级分类：诊所=1、门诊=2、医院=3。  生美没有二级分类
     *                      department             医美分类下的门诊和医院则需要填写科室
     *                      medicalPracticeLicense 医美分类必须上传医疗执业许可证（medicalPracticeLicenseImg）
     *                      mainProduct            主打项目（mainpro）
     *                      fax                    传真
     *                      profile                公司简介
     *                  }
     * @param headers   HttpHeaders
     * @return ClubPo
     */
    @PostMapping("/club/upgrade")
    public ResponseJson<ClubPo> clubUpgrade(ClubUpgradeDto upgradeDto, @RequestHeader HttpHeaders headers){
        return registerService.clubUpgrade(upgradeDto, headers);
    }

    /**
     * PC端供应商注册
     *
     * spi旧接口：supplier/register
     *
     * @param shopRegisterDto ShopRegisterDto{
     *                        source                注册来源: 0网站 1小程序
     *                        name                  组织名称
     *                        sName                 供应商公司简称(sname)
     *                        bindMobile            企业绑定手机号
     *                        email                 邮箱
     *                        password              密码
     *                        linkMan               联系人
     *                        provinceId            省(provinceID)
     *                        cityId                市(cityID)
     *                        townId                所在县区Id(townID)
     *                        address               地址
     *                        socialCreditCode      统一社会信用代码
     *                        businessLicenseImage  营业执照
     *                        firstShopType         医疗=1和非医疗=2
     *                        secondShopType        医疗的二级分类 一类器械=1、二类器械 =2、三类器械=3、其他=4 /// 1和非医疗没有二级分类
     *                        mainPro               主打项目(mainpro)
     *                    }
     *
     * @param passWordConfirm 用户确认密码
     * @param smsCode         短信验证码(旧：activationCode)
     * @param isAgreed        是否同意勾选同意协议，1是，其他否
     * @param headers         HttpHeaders
     * @return ShopPo
     */
    @Idempotent(prefix="idempotent_shop", keys={"#shopRegisterDto"}, expire=5)
    @PostMapping("/shop/pc")
    public ResponseJson<ShopPo> shopPcRegister(ShopRegisterDto shopRegisterDto,
                                               String passWordConfirm,
                                               String smsCode,
                                               Integer isAgreed,
                                               @RequestHeader HttpHeaders headers) {
        return registerService.shopRegister(shopRegisterDto, passWordConfirm, smsCode, isAgreed, headers);
    }

    /**
     * 小程序端分步供应商注册
     *
     * spi旧接口：supplier/appletsRegistered
     *
     * @param shopRegisterDto ShopRegisterDto{
     *                        source                注册来源: 0网站 1小程序
     *                        name                  组织名称
     *                        sName                 供应商公司简称(sname)
     *                        bindMobile            企业绑定手机号
     *                        email                 邮箱
     *                        password              密码
     *                        linkMan               联系人
     *                        provinceId            省(provinceID)
     *                        cityId                市(cityID)
     *                        townId                所在县区Id(townID)
     *                        address               地址
     *                        socialCreditCode      统一社会信用代码
     *                        businessLicenseImage  营业执照
     *                        firstShopType         医疗=1和非医疗=2
     *                        secondShopType        医疗的二级分类 一类器械=1、二类器械 =2、三类器械=3、其他=4 /// 1和非医疗没有二级分类
     *                        mainPro               主打项目(mainpro)
     *                    }
     * @param passWordConfirm 用户确认密码
     * @param smsCode         短信验证码(旧：activationCode)
     * @param isAgreed        是否同意勾选同意协议，1是，其他否
     * @param whichStep       注册步数
     * @param headers         HttpHeaders
     * @return ShopPo
     */
    @Idempotent(prefix="idempotent_shop2", keys={"#shopRegisterDto"}, expire=5)
    @PostMapping("/shop/applets")
    public ResponseJson shopAppletsRegister(ShopRegisterDto shopRegisterDto,
                                            String passWordConfirm,
                                            String smsCode,
                                            Integer isAgreed,
                                            Integer whichStep,
                                            @RequestHeader HttpHeaders headers) {
        return registerService.shopAppletsRegister(shopRegisterDto, passWordConfirm, smsCode, isAgreed, whichStep, headers);
    }
}



